@@ -11,11 +11,12 @@ use std::{
 };
 
 use crate::{
+    class::RClass,
     debug_assert_value,
     error::{protect, Error},
+    exception,
     object::Object,
     r_array::RArray,
-    r_class::RClass,
     ruby_sys::{
         rb_struct_aref, rb_struct_aset, rb_struct_define, rb_struct_getmember, rb_struct_members,
         rb_struct_size, ruby_value_type, VALUE,
@@ -87,6 +88,10 @@ impl RStruct {
         }
     }
 
+    pub(crate) unsafe fn from_rb_value_unchecked(val: VALUE) -> Self {
+        Self(NonZeroValue::new_unchecked(Value::new(val)))
+    }
+
     fn as_internal(self) -> NonNull<sys::RStruct> {
         // safe as inner value is NonZero
         unsafe { NonNull::new_unchecked(self.0.get().as_rb_value() as *mut _) }
@@ -131,11 +136,14 @@ impl RStruct {
             slice
                 .get(index)
                 .ok_or_else(|| {
-                    Error::index_error(format!(
-                        "offset {} too large for struct(size:{})",
-                        index,
-                        slice.len()
-                    ))
+                    Error::new(
+                        exception::index_error(),
+                        format!(
+                            "offset {} too large for struct(size:{})",
+                            index,
+                            slice.len()
+                        ),
+                    )
                 })
                 .and_then(|v| v.try_convert())
         }
@@ -245,10 +253,12 @@ impl TryConvert for RStruct {
     #[inline]
     fn try_convert(val: &Value) -> Result<Self, Error> {
         Self::from_value(*val).ok_or_else(|| {
-            Error::type_error(format!(
-                "no implicit conversion of {} into Struct",
-                unsafe { val.classname() },
-            ))
+            Error::new(
+                exception::type_error(),
+                format!("no implicit conversion of {} into Struct", unsafe {
+                    val.classname()
+                },),
+            )
         })
     }
 }
