@@ -3,6 +3,7 @@ use std::path::PathBuf;
 use crate::{
     debug_assert_value,
     error::{protect, Error},
+    exception,
     integer::Integer,
     r_array::RArray,
     r_hash::RHash,
@@ -212,7 +213,10 @@ where
         let array = RArray::try_convert(val)?;
         let slice = unsafe { array.as_slice() };
         if slice.len() != 1 {
-            return Err(Error::type_error("expected Array of length 1"));
+            return Err(Error::new(
+                exception::type_error(),
+                "expected Array of length 1",
+            ));
         }
         Ok((slice[0].try_convert()?,))
     }
@@ -230,7 +234,10 @@ where
         let array = RArray::try_convert(val)?;
         let slice = unsafe { array.as_slice() };
         if slice.len() != 2 {
-            return Err(Error::type_error("expected Array of length 2"));
+            return Err(Error::new(
+                exception::type_error(),
+                "expected Array of length 2",
+            ));
         }
         Ok((slice[0].try_convert()?, slice[1].try_convert()?))
     }
@@ -254,7 +261,10 @@ where
         let array = RArray::try_convert(val)?;
         let slice = unsafe { array.as_slice() };
         if slice.len() != 3 {
-            return Err(Error::type_error("expected Array of length 3"));
+            return Err(Error::new(
+                exception::type_error(),
+                "expected Array of length 3",
+            ));
         }
         Ok((
             slice[0].try_convert()?,
@@ -284,7 +294,10 @@ where
         let array = RArray::try_convert(val)?;
         let slice = unsafe { array.as_slice() };
         if slice.len() != 4 {
-            return Err(Error::type_error("expected Array of length 4"));
+            return Err(Error::new(
+                exception::type_error(),
+                "expected Array of length 4",
+            ));
         }
         Ok((
             slice[0].try_convert()?,
@@ -317,7 +330,10 @@ where
         let array = RArray::try_convert(val)?;
         let slice = unsafe { array.as_slice() };
         if slice.len() != 5 {
-            return Err(Error::type_error("expected Array of length 5"));
+            return Err(Error::new(
+                exception::type_error(),
+                "expected Array of length 5",
+            ));
         }
         Ok((
             slice[0].try_convert()?,
@@ -353,7 +369,10 @@ where
         let array = RArray::try_convert(val)?;
         let slice = unsafe { array.as_slice() };
         if slice.len() != 6 {
-            return Err(Error::type_error("expected Array of length 6"));
+            return Err(Error::new(
+                exception::type_error(),
+                "expected Array of length 6",
+            ));
         }
         Ok((
             slice[0].try_convert()?,
@@ -392,7 +411,10 @@ where
         let array = RArray::try_convert(val)?;
         let slice = unsafe { array.as_slice() };
         if slice.len() != 7 {
-            return Err(Error::type_error("expected Array of length 7"));
+            return Err(Error::new(
+                exception::type_error(),
+                "expected Array of length 7",
+            ));
         }
         Ok((
             slice[0].try_convert()?,
@@ -434,7 +456,10 @@ where
         let array = RArray::try_convert(val)?;
         let slice = unsafe { array.as_slice() };
         if slice.len() != 8 {
-            return Err(Error::type_error("expected Array of length 8"));
+            return Err(Error::new(
+                exception::type_error(),
+                "expected Array of length 8",
+            ));
         }
         Ok((
             slice[0].try_convert()?,
@@ -479,7 +504,10 @@ where
         let array = RArray::try_convert(val)?;
         let slice = unsafe { array.as_slice() };
         if slice.len() != 9 {
-            return Err(Error::type_error("expected Array of length 9"));
+            return Err(Error::new(
+                exception::type_error(),
+                "expected Array of length 9",
+            ));
         }
         Ok((
             slice[0].try_convert()?,
@@ -527,7 +555,10 @@ where
         let array = RArray::try_convert(val)?;
         let slice = unsafe { array.as_slice() };
         if slice.len() != 10 {
-            return Err(Error::type_error("expected Array of length 10"));
+            return Err(Error::new(
+                exception::type_error(),
+                "expected Array of length 10",
+            ));
         }
         Ok((
             slice[0].try_convert()?,
@@ -580,7 +611,10 @@ where
         let array = RArray::try_convert(val)?;
         let slice = unsafe { array.as_slice() };
         if slice.len() != 11 {
-            return Err(Error::type_error("expected Array of length 11"));
+            return Err(Error::new(
+                exception::type_error(),
+                "expected Array of length 11",
+            ));
         }
         Ok((
             slice[0].try_convert()?,
@@ -636,7 +670,10 @@ where
         let array = RArray::try_convert(val)?;
         let slice = unsafe { array.as_slice() };
         if slice.len() != 12 {
-            return Err(Error::type_error("expected Array of length 12"));
+            return Err(Error::new(
+                exception::type_error(),
+                "expected Array of length 12",
+            ));
         }
         Ok((
             slice[0].try_convert()?,
